@@ -1,6 +1,12 @@
 #ifndef LIBFILEZILLA_AIO_READER_HEADER
 #define LIBFILEZILLA_AIO_READER_HEADER
 
+/** \file
+ * \brief Classes for reading data asynchronously from files.
+ *
+ * Also has readers to get data from strings, views and buffers.
+ */
+
 #include "aio.hpp"
 #include "../file.hpp"
 #include "../thread_pool.hpp"
@@ -17,7 +23,7 @@ namespace fz {
  *
  * See the aio demo program for example usage.
  */
-class FZ_PUBLIC_SYMBOL reader_base : public aio_base, protected aio_waiter, public aio_waitable
+class FZ_PUBLIC_SYMBOL reader_base : public aio_base, public aio_waitable, public event_handler
 {
 public:
 	reader_base(reader_base const&) = delete;
@@ -51,7 +57,6 @@ public:
 	 * After getting aio_result::wait, do not call get_buffer again until after the passed
 	 * waiter got on_buffer_availability() invoked.
 	 */
-	std::pair<aio_result, buffer_lease> get_buffer(aio_waiter & h);
 	std::pair<aio_result, buffer_lease> get_buffer(event_handler & h);
 
 	bool error() const;
@@ -67,14 +72,16 @@ protected:
 	 * \sa reader_factory::multiple_buffer_usage()
 	 */
 	reader_base(std::wstring && name, aio_buffer_pool & pool, size_t max_buffers) noexcept
-	    : buffer_pool_(pool)
+	    : event_handler(pool.loop())
+	    , buffer_pool_(pool)
 	    , logger_(pool.logger())
 	    , name_(name)
 	    , max_buffers_(max_buffers ? max_buffers : 1)
 	{}
 
 	reader_base(std::wstring_view name, aio_buffer_pool & pool, size_t max_buffers) noexcept
-	    : buffer_pool_(pool)
+	    : event_handler(pool.loop())
+	    , buffer_pool_(pool)
 	    , logger_(pool.logger())
 	    , name_(name)
 	    , max_buffers_(max_buffers ? max_buffers : 1)
@@ -90,6 +97,10 @@ protected:
 
 	virtual void do_close(scoped_lock &) {}
 
+	virtual void operator()(event_base const&) override;
+	void on_buffer_availability(aio_waitable const* w);
+	virtual void do_on_buffer_availability(fz::scoped_lock & l, aio_waitable const* w) = 0;
+
 	mutable mutex mtx_;
 	aio_buffer_pool & buffer_pool_;
 	logger_interface & logger_;
@@ -107,6 +118,7 @@ protected:
 	bool get_buffer_called_{};
 	bool error_{};
 	bool eof_{};
+	bool waiting_{};
 };
 
 /// A reader factory
@@ -134,7 +146,7 @@ public:
 	 *
 	 * \c size can limit the amount of data the reader can returned. Note
 	 * that a size limit that exceeds the actual size will result in
-	 * reader_base::get_buffer evenutually returning an error.
+	 * reader_base::get_buffer eventually returning an error.
 	 */
 	virtual std::unique_ptr<reader_base> open(aio_buffer_pool & pool, uint64_t offset = 0, uint64_t size = reader_base::nosize, size_t max_buffers = 0) = 0;
 
@@ -240,7 +252,7 @@ private:
 	virtual void FZ_PRIVATE_SYMBOL do_close(scoped_lock & l) override;
 	virtual bool FZ_PRIVATE_SYMBOL do_seek(scoped_lock & l) override;
 
-	virtual void FZ_PRIVATE_SYMBOL on_buffer_availability(aio_waitable const* w) override;
+	virtual void FZ_PRIVATE_SYMBOL do_on_buffer_availability(scoped_lock & l, aio_waitable const* w) override;
 
 	void FZ_PRIVATE_SYMBOL entry();
 
@@ -287,7 +299,7 @@ private:
 	virtual void FZ_PRIVATE_SYMBOL do_close(scoped_lock & l) override;
 	virtual bool FZ_PRIVATE_SYMBOL do_seek(scoped_lock & l) override;
 
-	virtual void FZ_PRIVATE_SYMBOL on_buffer_availability(aio_waitable const* w) override;
+	virtual void FZ_PRIVATE_SYMBOL do_on_buffer_availability(scoped_lock & l, aio_waitable const* w) override;
 
 	std::string_view const view_;
 };
@@ -336,7 +348,7 @@ private:
 	virtual void FZ_PRIVATE_SYMBOL do_close(scoped_lock & l) override;
 	virtual bool FZ_PRIVATE_SYMBOL do_seek(scoped_lock & l) override;
 
-	virtual void FZ_PRIVATE_SYMBOL on_buffer_availability(aio_waitable const* w) override;
+	virtual void FZ_PRIVATE_SYMBOL do_on_buffer_availability(scoped_lock & l, aio_waitable const* w) override;
 
 	std::string const data_;
 };
