@@ -4,6 +4,7 @@
 #include "libfilezilla.hpp"
 
 #include <vector>
+#include <type_traits>
 
 /** \file
 * \brief Declares fz::buffer
@@ -25,6 +26,8 @@ namespace fz {
 class FZ_PUBLIC_SYMBOL buffer final
 {
 public:
+	typedef unsigned char value_type;
+
 	buffer() noexcept = default;
 
 	/// Initially reserves the passed capacity
@@ -42,6 +45,10 @@ public:
 	unsigned char const* get() const { return pos_; }
 	unsigned char* get() { return pos_; }
 
+	/// Same as get()
+	unsigned char* data() { return pos_; }
+	unsigned char const* data() const { return pos_; }
+
 	/** \brief Returns a writable buffer guaranteed to be large enough for write_size bytes, call add when done.
 	 *
 	 * The returned pointer is pointing just after the data already stored in the buffer.
@@ -65,6 +72,19 @@ public:
 	/// Increase size by the passed amount. Call this after having obtained a writable buffer with get(size_t write_size)
 	void add(size_t added);
 
+	/**
+	 * \brief Overload of add for signed types, only adds if value is positive.
+	 *
+	 * Does nothing on values <= 0, useful for directly passing the ssize_t result of
+	 * the system's recv() or read() functions.
+	 */
+	template<typename T, std::enable_if_t<std::is_signed_v<T>, int> = 0>
+	void add(T added) {
+		if (added > 0) {
+			add(static_cast<size_t>(added));
+		}
+	}
+
 	/** \brief Removes consumed bytes from the beginning of the buffer.
 	 *
 	 * Undefined if consumed > size()
@@ -84,8 +104,28 @@ public:
 	 */
 	void append(unsigned char const* data, size_t len);
 	void append(std::string_view const& str);
+	void append(std::basic_string_view<uint8_t> const& data);
 	void append(std::vector<uint8_t> const& data);
+	void append(fz::buffer const& b);
 	void append(unsigned char v);
+	void append(size_t len, unsigned char c);
+
+	buffer& operator+=(unsigned char v) {
+		append(v);
+		return *this;
+	}
+	buffer& operator+=(std::string_view const& str) {
+		append(str);
+		return *this;
+	}
+	buffer& operator+=(std::vector<uint8_t> const& data) {
+		append(data);
+		return *this;
+	}
+	buffer& operator+=(fz::buffer const& b) {
+		append(b);
+		return *this;
+	}
 
 	bool empty() const { return size_ == 0; }
 	explicit operator bool() const {
@@ -108,6 +148,10 @@ public:
 	}
 
 	std::string_view to_view() const;
+
+	void wipe();
+	void wipe_unused();
+
 private:
 
 	// Invariants:
@@ -121,6 +165,14 @@ private:
 	size_t capacity_{};
 };
 
+inline void FZ_PUBLIC_SYMBOL wipe(buffer & b) {
+	b.wipe();
+}
+
+inline void FZ_PUBLIC_SYMBOL wipe_unused(buffer & b) {
+	b.wipe_unused();
+}
+
 }
 
 #endif
