@@ -18,12 +18,15 @@ enum class hash_algorithm
 	md5, // insecure
 	sha1, // insecure
 	sha256,
+	sha384,
 	sha512
 };
 
 enum class hmac_algorithm
 {
-	sha256
+	sha1, // insecure
+	sha256,
+	sha512
 };
 
 /// Returns digest size in bytes
@@ -38,6 +41,7 @@ public:
 	/// Creates an initialized accumulator for the passed algorithm
 	hash_accumulator(hash_algorithm algorithm);
 	hash_accumulator(hmac_algorithm algorithm, std::vector<uint8_t> const& key);
+	hash_accumulator(hmac_algorithm algorithm, std::string_view const& key);
 	~hash_accumulator();
 
 	hash_accumulator(hash_accumulator const&) = delete;
@@ -56,6 +60,12 @@ public:
 		update(&in, 1);
 	}
 
+	/// If platform is little-endian, converts to big-endian, aka network byte order, before feeding the hash
+	void update_uint32_be(uint32_t v);
+
+	/// Also feeds length, as uint32 in network byte order, to the hash
+	void update_with_length(std::string_view const& data);
+
 	/// Returns the raw digest and reinitializes the accumulator
 	std::vector<uint8_t> digest();
 	void digest(uint8_t* out, size_t s);
@@ -93,6 +103,10 @@ std::vector<uint8_t> FZ_PUBLIC_SYMBOL md5(std::vector<uint8_t> const& data);
 std::vector<uint8_t> FZ_PUBLIC_SYMBOL sha256(std::string_view const& data);
 std::vector<uint8_t> FZ_PUBLIC_SYMBOL sha256(std::vector<uint8_t> const& data);
 
+/// \brief Standard SHA512
+std::vector<uint8_t> FZ_PUBLIC_SYMBOL sha512(std::string_view const& data);
+std::vector<uint8_t> FZ_PUBLIC_SYMBOL sha512(std::vector<uint8_t> const& data);
+
 /** \brief Standard HMAC using SHA1
  *
  * While HMAC-SHA1 (as opposed to plain SHA1) is still considered secure in 2021, avoid using this for new things
