@@ -5,19 +5,33 @@
 * \brief Declares \ref fz::impersonation_token
 */
 
-#include "string.hpp"
-
 #include <memory>
 #include <functional>
 
+#include "string.hpp"
+#include "logger.hpp"
+
+#ifdef FZ_WINDOWS
+#include "glue/windows.hpp"
+#endif
+
 namespace fz {
 
-#if !FZ_WINDOWS
-enum class impersonation_flag
+class logger_interface;
+
+struct impersonation_options
 {
-	pwless /// Impersonate as any user without checking credentials
-};
+	/// Impersonate as any user without checking credentials
+	struct pwless_type{};
+	static constexpr pwless_type pwless{};
+
+#if FZ_WINDOWS
+	bool drop_admin_privileges = true;
+#else
+	/// If set, overrides the group id.
+	fz::native_string group;
 #endif
+};
 
 class impersonation_token_impl;
 
@@ -37,13 +51,9 @@ public:
 	impersonation_token(impersonation_token&&) noexcept;
 	impersonation_token& operator=(impersonation_token&&) noexcept;
 
-	/// Creates an impersonation token, verifying credentials in the proceess
-	explicit impersonation_token(fz::native_string const& username, fz::native_string const& password);
-
-#if !FZ_WINDOWS
-	/// Doesn't verify credentials
-	explicit impersonation_token(fz::native_string const& username, impersonation_flag flag, fz::native_string const& group = {});
-#endif
+	/// Creates an impersonation token, verifying credentials in the process.
+	explicit impersonation_token(fz::native_string const& username, fz::native_string const &password, fz::logger_interface& logger = get_null_logger(), impersonation_options const& opts = {});
+	explicit impersonation_token(fz::native_string const& username, impersonation_options::pwless_type, fz::logger_interface& logger = get_null_logger(), impersonation_options const& opts = {});
 
 	~impersonation_token() noexcept;
 
@@ -63,7 +73,12 @@ public:
 	/// For std::hash
 	std::size_t hash() const noexcept;
 
+	/// A opaque unique identifier
+	std::string uid() const;
+
 private:
+	impersonation_token(fz::native_string const& username, fz::native_string const *password, fz::logger_interface& logger = get_null_logger(), impersonation_options const& opts = {});
+
 	friend class impersonation_token_impl;
 	std::unique_ptr<impersonation_token_impl> impl_;
 };
@@ -76,6 +91,10 @@ bool FZ_PUBLIC_SYMBOL set_process_impersonation(impersonation_token const& token
 /// Returns the username the calling thread is running under
 native_string FZ_PUBLIC_SYMBOL current_username();
 
+/// \returns An opaque user unique identifier, if the user exists and no error occurs, an empty string otherwise.
+/// \note On some systems, different user names can resolve to the same unique identifier.
+std::string FZ_PUBLIC_SYMBOL get_user_uid(native_string const& username);
+
 }
 
 namespace std {
