@@ -445,6 +445,12 @@ public:
 			return s_.empty();
 		}
 
+		constexpr iterator& operator =(sentinel)
+		{
+			s_ = {};
+			return *this;
+		}
+
 		constexpr bool operator ==(iterator const& op) const
 		{
 			return s_.size() == op.s_.size();
